@@ -30,7 +30,7 @@ from . import weights
 from . import resolution
 from . import resolution2d
 from .details import make_kernel_args, dispersion_mesh
-from .modelinfo import DEFAULT_BACKGROUND
+from .product import RADIUS_MODE_ID
 
 # pylint: disable=unused-import
 try:
@@ -58,54 +58,36 @@ def call_kernel(calculator, pars, cutoff=0., mono=False):
     *mono* is True if polydispersity should be set to none on all parameters.
     """
     mesh = get_mesh(calculator.info, pars, dim=calculator.dim, mono=mono)
-    #print("pars", list(zip(*mesh))[0])
+    #print("in call_kernel: pars:", list(zip(*mesh))[0])
     call_details, values, is_magnetic = make_kernel_args(calculator, mesh)
-    #print("values:", values)
+    #print("in call_kernel: values:", values)
     return calculator(call_details, values, cutoff, is_magnetic)
 
-def call_ER(model_info, pars):
-    # type: (ModelInfo, ParameterSet) -> float
+def call_Fq(calculator, pars, cutoff=0., mono=False):
+    # type: (Kernel, ParameterSet, float, bool) -> np.ndarray
     """
-    Call the model ER function using *values*.
+    Like :func:`call_kernel`, but returning F, F^2, R_eff, V_shell, V_form/V_shell.
 
-    *model_info* is either *model.info* if you have a loaded model,
-    or *kernel.info* if you have a model kernel prepared for evaluation.
+    For solid objects V_shell is equal to V_form and the volume ratio is 1.
+
+    Use parameter *radius_effective_mode* to select the effective radius
+    calculation to use amongst the *radius_effective_modes* list given in the
+    model.
     """
-    if model_info.ER is None:
-        return 1.0
-    elif not model_info.parameters.form_volume_parameters:
-        # handle the case where ER is provided but model is not polydisperse
-        return model_info.ER()
-    else:
-        value, weight = _vol_pars(model_info, pars)
-        individual_radii = model_info.ER(*value)
-        return np.sum(weight*individual_radii) / np.sum(weight)
+    R_eff_type = int(pars.pop(RADIUS_MODE_ID, 1.0))
+    mesh = get_mesh(calculator.info, pars, dim=calculator.dim, mono=mono)
+    #print("in call_Fq: pars", list(zip(*mesh))[0])
+    call_details, values, is_magnetic = make_kernel_args(calculator, mesh)
+    #print("in call_Fq: values:", values)
+    return calculator.Fq(call_details, values, cutoff, is_magnetic, R_eff_type)
 
-
-def call_VR(model_info, pars):
-    # type: (ModelInfo, ParameterSet) -> float
-    """
-    Call the model VR function using *pars*.
-
-    *model_info* is either *model.info* if you have a loaded model,
-    or *kernel.info* if you have a model kernel prepared for evaluation.
-    """
-    if model_info.VR is None:
-        return 1.0
-    elif not model_info.parameters.form_volume_parameters:
-        # handle the case where ER is provided but model is not polydisperse
-        return model_info.VR()
-    else:
-        value, weight = _vol_pars(model_info, pars)
-        whole, part = model_info.VR(*value)
-        return np.sum(weight*part)/np.sum(weight*whole)
-
-
-def call_profile(model_info, **pars):
-    # type: (ModelInfo, ...) -> Tuple[np.ndarray, np.ndarray, Tuple[str, str]]
+def call_profile(model_info, pars=None):
+    # type: (ModelInfo, ParameterSet) -> Tuple[np.ndarray, np.ndarray, Tuple[str, str]]
     """
     Returns the profile *x, y, (xlabel, ylabel)* representing the model.
     """
+    if pars is None:
+        pars = {}
     args = {}
     for p in model_info.parameters.kernel_parameters:
         if p.length > 1:
@@ -136,7 +118,7 @@ def get_mesh(model_info, values, dim='1d', mono=False):
     else:
         active = lambda name: True
 
-    #print("pars",[p.id for p in parameters.call_parameters])
+    #print("in get_mesh: pars:",[p.id for p in parameters.call_parameters])
     mesh = [_get_par_weights(p, values, active(p.name))
             for p in parameters.call_parameters]
     return mesh
@@ -304,6 +286,7 @@ class DataMixin(object):
         self._kernel = None
         self.Iq, self.dIq, self.index = Iq, dIq, index
         self.resolution = res
+        self.results = None  # type: Optional[Callable[[], OrderedDict]
 
     def _set_data(self, Iq, noise=None):
         # type: (np.ndarray, Optional[float]) -> None
@@ -346,11 +329,13 @@ class DataMixin(object):
             self._kernel = self._model.make_kernel(kernel_inputs)
 
         # Need to pull background out of resolution for multiple scattering
-        background = pars.get('background', DEFAULT_BACKGROUND)
+        default_background = self._model.info.parameters.common_parameters[1].default
+        background = pars.get('background', default_background)
         pars = pars.copy()
         pars['background'] = 0.
 
         Iq_calc = call_kernel(self._kernel, pars, cutoff=cutoff)
+        self.results = getattr(self._kernel, 'results', None)
         # Storing the calculated Iq values so that they can be plotted.
         # Only applies to oriented USANS data for now.
         # TODO: extend plotting of calculate Iq to other measurement types
@@ -399,7 +384,7 @@ class DirectModel(DataMixin):
         """
         Generate a plottable profile.
         """
-        return call_profile(self.model.info, **pars)
+        return call_profile(self.model.info, pars)
 
 def main():
     # type: () -> None
@@ -415,22 +400,19 @@ def main():
         sys.exit(1)
     model_name = sys.argv[1]
     call = sys.argv[2].upper()
-    if call != "ER_VR":
-        try:
-            values = [float(v) for v in call.split(',')]
-        except ValueError:
-            values = []
-        if len(values) == 1:
-            q, = values
-            data = empty_data1D([q])
-        elif len(values) == 2:
-            qx, qy = values
-            data = empty_data2D([qx], [qy])
-        else:
-            print("use q or qx,qy or ER or VR")
-            sys.exit(1)
+    try:
+        values = [float(v) for v in call.split(',')]
+    except ValueError:
+        values = []
+    if len(values) == 1:
+        q, = values
+        data = empty_data1D([q])
+    elif len(values) == 2:
+        qx, qy = values
+        data = empty_data2D([qx], [qy])
     else:
-        data = empty_data1D([0.001])  # Data not used in ER/VR
+        print("use q or qx,qy")
+        sys.exit(1)
 
     model_info = load_model_info(model_name)
     model = build_model(model_info)
@@ -438,13 +420,8 @@ def main():
     pars = dict((k, (float(v) if not k.endswith("_pd_type") else v))
                 for pair in sys.argv[3:]
                 for k, v in [pair.split('=')])
-    if call == "ER_VR":
-        ER = call_ER(model_info, pars)
-        VR = call_VR(model_info, pars)
-        print(ER, VR)
-    else:
-        Iq = calculator(**pars)
-        print(Iq[0])
+    Iq = calculator(**pars)
+    print(Iq[0])
 
 if __name__ == "__main__":
     main()
