@@ -4,21 +4,24 @@ Run model unit tests.
 
 Usage::
 
-    python -m sasmodels.model_test [opencl|dll|opencl_and_dll] model1 model2 ...
+    python -m sasmodels.model_test [opencl|cuda|dll|all] model1 model2 ...
 
-    if model1 is 'all', then all except the remaining models will be tested
+If model1 is 'all', then all except the remaining models will be tested.
+Subgroups are also possible, such as 'py', 'single' or '1d'.  See
+:func:`core.list_models` for details.
 
 Each model is tested using the default parameters at q=0.1, (qx, qy)=(0.1, 0.1),
-and the ER and VR are computed.  The return values at these points are not
-considered.  The test is only to verify that the models run to completion,
-and do not produce inf or NaN.
+and Fq is called to make sure R_eff, volume and volume ratio are computed.
+The return values at these points are not considered.  The test is only to
+verify that the models run to completion, and do not produce inf or NaN.
 
 Tests are defined with the *tests* attribute in the model.py file.  *tests*
 is a list of individual tests to run, where each test consists of the
 parameter values for the test, the q-values and the expected results.  For
-the effective radius test, the q-value should be 'ER'.  For the VR test,
-the q-value should be 'VR'.  For 1-D tests, either specify the q value or
-a list of q-values, and the corresponding I(q) value, or list of I(q) values.
+the effective radius test and volume ratio tests, use the extended output
+form, which checks each output of kernel.Fq. For 1-D tests, either specify
+the q value or a list of q-values, and the corresponding I(q) value, or
+list of I(q) values.
 
 That is::
 
@@ -31,8 +34,7 @@ That is::
         [ {parameters}, [(qx1, qy1), (qx2, qy2), ...],
                         [I(qx1, qy1), I(qx2, qy2), ...]],
 
-        [ {parameters}, 'ER', ER(pars) ],
-        [ {parameters}, 'VR', VR(pars) ],
+        [ {parameters}, q, F(q), F^2(q), R_eff, V, V_r ],
         ...
     ]
 
@@ -44,6 +46,7 @@ Precision defaults to 5 digits (relative).
 """
 from __future__ import print_function
 
+import argparse
 import sys
 import unittest
 import traceback
@@ -57,12 +60,13 @@ except ImportError:
 
 import numpy as np  # type: ignore
 
-from . import core
 from .core import list_models, load_model_info, build_model
-from .direct_model import call_kernel, call_ER, call_VR
+from .direct_model import call_kernel, call_Fq
 from .exception import annotate_exception
 from .modelinfo import expand_pars
 from .kernelcl import use_opencl
+from .kernelcuda import use_cuda
+from . import product
 
 # pylint: disable=unused-import
 try:
@@ -79,18 +83,19 @@ def make_suite(loaders, models):
     """
     Construct the pyunit test suite.
 
-    *loaders* is the list of kernel drivers to use, which is one of
-    *["dll", "opencl"]*, *["dll"]* or *["opencl"]*.  For python models,
-    the python driver is always used.
+    *loaders* is the list of kernel drivers to use (dll, opencl or cuda).
+    For python model the python driver is always used.
 
     *models* is the list of models to test, or *["all"]* to test all models.
     """
     suite = unittest.TestSuite()
 
-    if models[0] in core.KINDS:
+    try:
+        # See if the first model parses as a model group
+        group = list_models(models[0])
         skip = models[1:]
-        models = list_models(models[0])
-    else:
+        models = group
+    except Exception:
         skip = []
     for model_name in models:
         if model_name not in skip:
@@ -130,10 +135,10 @@ def _add_model_to_suite(loaders, suite, model_info):
         test_name = "%s-python"%model_info.name
         test_method_name = "test_%s_python" % model_info.id
         test = ModelTestCase(test_name, model_info,
-                                test_method_name,
-                                platform="dll",  # so that
-                                dtype="double",
-                                stash=stash)
+                             test_method_name,
+                             platform="dll",  # so that
+                             dtype="double",
+                             stash=stash)
         suite.addTest(test)
     else:   # kernel implemented in C
 
@@ -142,10 +147,10 @@ def _add_model_to_suite(loaders, suite, model_info):
             test_name = "%s-dll"%model_info.name
             test_method_name = "test_%s_dll" % model_info.id
             test = ModelTestCase(test_name, model_info,
-                                    test_method_name,
-                                    platform="dll",
-                                    dtype="double",
-                                    stash=stash)
+                                 test_method_name,
+                                 platform="dll",
+                                 dtype="double",
+                                 stash=stash)
             suite.addTest(test)
 
         # test using opencl if desired and available
@@ -157,9 +162,24 @@ def _add_model_to_suite(loaders, suite, model_info):
             # single precision.  The choice is determined by the
             # presence of *single=False* in the model file.
             test = ModelTestCase(test_name, model_info,
-                                    test_method_name,
-                                    platform="ocl", dtype=None,
-                                    stash=stash)
+                                 test_method_name,
+                                 platform="ocl", dtype=None,
+                                 stash=stash)
+            #print("defining", test_name)
+            suite.addTest(test)
+
+        # test using cuda if desired and available
+        if 'cuda' in loaders and use_cuda():
+            test_name = "%s-cuda" % model_info.id
+            test_method_name = "test_%s_cuda" % model_info.id
+            # Using dtype=None so that the models that are only
+            # correct for double precision are not tested using
+            # single precision.  The choice is determined by the
+            # presence of *single=False* in the model file.
+            test = ModelTestCase(test_name, model_info,
+                                 test_method_name,
+                                 platform="cuda", dtype=None,
+                                 stash=stash)
             #print("defining", test_name)
             suite.addTest(test)
 
@@ -201,18 +221,36 @@ def _hide_model_case_from_nose():
                 # test vector form
                 ({}, [0.001, 0.01, 0.1], [None]*3),
                 ({}, [(0.1, 0.1)]*2, [None]*2),
-                # test that ER/VR will run if they exist
-                ({}, 'ER', None),
-                ({}, 'VR', None),
+                # test that Fq will run, and return R_eff, V, V_r
+                ({}, 0.1, None, None, None, None, None),
                 ]
             tests = smoke_tests
             #tests = []
             if self.info.tests is not None:
                 tests += self.info.tests
+            S_tests = [test for test in tests if '@S' in test[0]]
+            P_tests = [test for test in tests if '@S' not in test[0]]
             try:
                 model = build_model(self.info, dtype=self.dtype,
                                     platform=self.platform)
-                results = [self.run_one(model, test) for test in tests]
+                results = [self.run_one(model, test) for test in P_tests]
+                for test in S_tests:
+                    # pull the S model name out of the test defn
+                    pars = test[0].copy()
+                    s_name = pars.pop('@S')
+                    ps_test = [pars] + list(test[1:])
+                    #print("PS TEST PARAMS!!!",ps_test)
+                    # build the P@S model
+                    s_info = load_model_info(s_name)
+                    ps_info = product.make_product_info(self.info, s_info)
+                    ps_model = build_model(ps_info, dtype=self.dtype,
+                                           platform=self.platform)
+                    # run the tests
+                    #self.info = ps_model.info
+                    #print("SELF.INFO PARAMS!!!",[p.id for p in self.info.parameters.call_parameters])
+                    #print("PS MODEL PARAMETERS:",[p.id for p in ps_model.info.parameters.call_parameters])
+                    results.append(self.run_one(ps_model, ps_test))
+
                 if self.stash:
                     for test, target, actual in zip(tests, self.stash[0], results):
                         assert np.all(abs(target-actual) < 5e-5*abs(actual)), \
@@ -222,7 +260,7 @@ def _hide_model_case_from_nose():
                     self.stash.append(results)
 
                 # Check for missing tests.  Only do so for the "dll" tests
-                # to reduce noise from both opencl and dll, and because
+                # to reduce noise from both opencl and cuda, and because
                 # python kernels use platform="dll".
                 if self.platform == "dll":
                     missing = []
@@ -237,9 +275,7 @@ def _hide_model_case_from_nose():
 
         def _find_missing_tests(self):
             # type: () -> None
-            """make sure there are 1D, 2D, ER and VR tests as appropriate"""
-            model_has_VR = callable(self.info.VR)
-            model_has_ER = callable(self.info.ER)
+            """make sure there are 1D and 2D tests as appropriate"""
             model_has_1D = True
             model_has_2D = any(p.type == 'orientation'
                                for p in self.info.parameters.kernel_parameters)
@@ -247,8 +283,6 @@ def _hide_model_case_from_nose():
             # Lists of tests that have a result that is not None
             single = [test for test in self.info.tests
                       if not isinstance(test[2], list) and test[2] is not None]
-            tests_has_VR = any(test[1] == 'VR' for test in single)
-            tests_has_ER = any(test[1] == 'ER' for test in single)
             tests_has_1D_single = any(isinstance(test[1], float) for test in single)
             tests_has_2D_single = any(isinstance(test[1], tuple) for test in single)
 
@@ -261,10 +295,6 @@ def _hide_model_case_from_nose():
                                         for test in multiple)
 
             missing = []
-            if model_has_VR and not tests_has_VR:
-                missing.append("VR")
-            if model_has_ER and not tests_has_ER:
-                missing.append("ER")
             if model_has_1D and not (tests_has_1D_single or tests_has_1D_multiple):
                 missing.append("1D")
             if model_has_2D and not (tests_has_2D_single or tests_has_2D_multiple):
@@ -275,9 +305,10 @@ def _hide_model_case_from_nose():
         def run_one(self, model, test):
             # type: (KernelModel, TestCondition) -> None
             """Run a single test case."""
-            user_pars, x, y = test
-            pars = expand_pars(self.info.parameters, user_pars)
-            invalid = invalid_pars(self.info.parameters, pars)
+            user_pars, x, y = test[:3]
+            #print("PS MODEL PARAMETERS:",[p.id for p in model.info.parameters.call_parameters])
+            pars = expand_pars(model.info.parameters, user_pars)
+            invalid = invalid_pars(model.info.parameters, pars)
             if invalid:
                 raise ValueError("Unknown parameters in test: " + ", ".join(invalid))
 
@@ -288,39 +319,45 @@ def _hide_model_case_from_nose():
 
             self.assertEqual(len(y), len(x))
 
-            if x[0] == 'ER':
-                actual = np.array([call_ER(model.info, pars)])
-            elif x[0] == 'VR':
-                actual = np.array([call_VR(model.info, pars)])
-            elif isinstance(x[0], tuple):
+            if isinstance(x[0], tuple):
                 qx, qy = zip(*x)
                 q_vectors = [np.array(qx), np.array(qy)]
-                kernel = model.make_kernel(q_vectors)
-                actual = call_kernel(kernel, pars)
             else:
                 q_vectors = [np.array(x)]
-                kernel = model.make_kernel(q_vectors)
+
+            kernel = model.make_kernel(q_vectors)
+            if len(test) == 3:
                 actual = call_kernel(kernel, pars)
+                self._check_vectors(x, y, actual, 'I')
+                return actual
+            else:
+                y1 = y
+                y2 = test[3] if isinstance(test[3], list) else [test[3]]
+                F, Fsq, R_eff, volume, volume_ratio = call_Fq(kernel, pars)
+                if F is not None:  # F is none for models with Iq instead of Fq
+                    self._check_vectors(x, y1, F, 'F')
+                self._check_vectors(x, y2, Fsq, 'F^2')
+                self._check_scalar(test[4], R_eff, 'R_eff')
+                self._check_scalar(test[5], volume, 'volume')
+                self._check_scalar(test[6], volume_ratio, 'form:shell ratio')
+                return Fsq
 
-            self.assertTrue(len(actual) > 0)
-            self.assertEqual(len(y), len(actual))
+        def _check_scalar(self, target, actual, name):
+            self.assertTrue(is_near(target, actual, 5),
+                            '%s: expected:%s; actual:%s'
+                            % (name, target, actual))
 
-            for xi, yi, actual_yi in zip(x, y, actual):
-                if yi is None:
-                    # smoke test --- make sure it runs and produces a value
-                    self.assertTrue(not np.isnan(actual_yi),
-                                    'invalid f(%s): %s' % (xi, actual_yi))
-                elif np.isnan(yi):
-                    self.assertTrue(np.isnan(actual_yi),
-                                    'f(%s): expected:%s; actual:%s'
-                                    % (xi, yi, actual_yi))
-                else:
-                    # is_near does not work for infinite values, so also test
-                    # for exact values.  Note that this will not
-                    self.assertTrue(yi == actual_yi or is_near(yi, actual_yi, 5),
-                                    'f(%s); expected:%s; actual:%s'
-                                    % (xi, yi, actual_yi))
-            return actual
+        def _check_vectors(self, x, target, actual, name='I'):
+            self.assertTrue(len(actual) > 0,
+                            '%s(...) expected return'%name)
+            if target is None:
+                return
+            self.assertEqual(len(target), len(actual),
+                             '%s(...) returned wrong length'%name)
+            for xi, yi, actual_yi in zip(x, target, actual):
+                self.assertTrue(is_near(yi, actual_yi, 5),
+                                '%s(%s): expected:%s; actual:%s'
+                                % (name, xi, target, actual))
 
     return ModelTestCase
 
@@ -332,6 +369,12 @@ def invalid_pars(partable, pars):
     names = set(p.id for p in partable.call_parameters)
     invalid = []
     for par in sorted(pars.keys()):
+        # Ignore the R_eff mode parameter when checking for valid parameters.
+        # It is an allowed parameter for a model even though it does not exist
+        # in the parameter table.  The call_Fq() function pops it from the
+        # parameter list and sends it directly to kernel.Fq().
+        if par == product.RADIUS_MODE_ID:
+            continue
         parts = par.split('_pd')
         if len(parts) > 1 and parts[1] not in ("", "_n", "nsigma", "type"):
             invalid.append(par)
@@ -345,13 +388,42 @@ def is_near(target, actual, digits=5):
     # type: (float, float, int) -> bool
     """
     Returns true if *actual* is within *digits* significant digits of *target*.
+
+    *taget* zero and inf should match *actual* zero and inf.  If you want to
+    accept eps for zero, choose a value such as 1e-10, which must match up to
+    +/- 1e-15 when *digits* is the default value of 5.
+
+    If *target* is None, then just make sure that *actual* is not NaN.
+
+    If *target* is NaN, make sure *actual* is NaN.
     """
-    import math
-    shift = 10**math.ceil(math.log10(abs(target)))
-    return abs(target-actual)/shift < 1.5*10**-digits
+    if target is None:
+        # target is None => actual cannot be NaN
+        return not np.isnan(actual)
+    elif target == 0.:
+        # target is 0. => actual must be 0.
+        # Note: if small values are allowed, then use maybe test zero against eps instead?
+        return actual == 0.
+    elif np.isfinite(target):
+        shift = np.ceil(np.log10(abs(target)))
+        return abs(target-actual) < 1.5*10**(shift-digits)
+    elif target == actual:
+        # target is inf => actual must be inf of same sign
+        return True
+    else:
+        # target is NaN => actual must be NaN
+        return np.isnan(target) == np.isnan(actual)
 
 # CRUFT: old interface; should be deprecated and removed
 def run_one(model_name):
+    # type: (str) -> str
+    """
+    [Deprecated] Run the tests associated with *model_name*.
+
+    Use the following instead::
+
+        succss, output = check_model(load_model_info(model_name))
+    """
     # msg = "use check_model(model_info) rather than run_one(model_name)"
     # warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
     try:
@@ -360,11 +432,11 @@ def run_one(model_name):
         output = traceback.format_exc()
         return output
 
-    success, output = check_model(model_info)
+    _, output = check_model(model_info)
     return output
 
 def check_model(model_info):
-    # type: (ModelInfo) -> str
+    # type: (ModelInfo) -> Tuple[bool, str]
     """
     Run the tests for a single model, capturing the output.
 
@@ -382,7 +454,7 @@ def check_model(model_info):
     result = TextTestResult(stream, descriptions, verbosity)
 
     # Build a test suite containing just the model
-    loaders = ['opencl'] if use_opencl() else ['dll']
+    loaders = ['opencl' if use_opencl() else 'cuda' if use_cuda() else 'dll']
     suite = unittest.TestSuite()
     _add_model_to_suite(loaders, suite, model_info)
 
@@ -418,61 +490,6 @@ def check_model(model_info):
     return result.wasSuccessful(), output
 
 
-def main(*models):
-    # type: (*str) -> int
-    """
-    Run tests given is models.
-
-    Returns 0 if success or 1 if any tests fail.
-    """
-    try:
-        from xmlrunner import XMLTestRunner as TestRunner
-        test_args = {'output': 'logs'}
-    except ImportError:
-        from unittest import TextTestRunner as TestRunner
-        test_args = {}
-
-    if models and models[0] == '-v':
-        verbosity = 2
-        models = models[1:]
-    else:
-        verbosity = 1
-    if models and models[0] == 'opencl':
-        if not use_opencl():
-            print("opencl is not available")
-            return 1
-        loaders = ['opencl']
-        models = models[1:]
-    elif models and models[0] == 'dll':
-        # TODO: test if compiler is available?
-        loaders = ['dll']
-        models = models[1:]
-    elif models and models[0] == 'opencl_and_dll':
-        loaders = ['opencl', 'dll'] if use_opencl() else ['dll']
-        models = models[1:]
-    else:
-        loaders = ['opencl', 'dll'] if use_opencl() else ['dll']
-    if not models:
-        print("""\
-usage:
-  python -m sasmodels.model_test [-v] [opencl|dll] model1 model2 ...
-
-If -v is included on the command line, then use verbose output.
-
-If neither opencl nor dll is specified, then models will be tested with
-both OpenCL and dll; the compute target is ignored for pure python models.
-
-If model1 is 'all', then all except the remaining models will be tested.
-
-""")
-
-        return 1
-
-    runner = TestRunner(verbosity=verbosity, **test_args)
-    result = runner.run(make_suite(loaders, models))
-    return 1 if result.failures or result.errors else 0
-
-
 def model_tests():
     # type: () -> Iterator[Callable[[], None]]
     """
@@ -480,9 +497,13 @@ def model_tests():
 
     Run "nosetests sasmodels" on the command line to invoke it.
     """
-    loaders = ['opencl', 'dll'] if use_opencl() else ['dll']
+    loaders = ['dll']
+    if use_opencl():
+        loaders.append('opencl')
+    if use_cuda():
+        loaders.append('cuda')
     tests = make_suite(loaders, ['all'])
-    def build_test(test):
+    def _build_test(test):
         # In order for nosetest to show the test name, wrap the test.run_all
         # instance in function that takes the test name as a parameter which
         # will be displayed when the test is run.  Do this as a function so
@@ -500,8 +521,63 @@ def model_tests():
         #     return lambda name: test.run_all(), test.test_name
 
     for test in tests:
-        yield build_test(test)
+        yield _build_test(test)
+
+
+def main():
+    # type: (*str) -> int
+    """
+    Run tests given is models.
+
+    Returns 0 if success or 1 if any tests fail.
+    """
+    try:
+        from xmlrunner import XMLTestRunner as TestRunner
+        test_args = {'output': 'logs'}
+    except ImportError:
+        from unittest import TextTestRunner as TestRunner
+        test_args = {}
+
+    parser = argparse.ArgumentParser(description="Test SasModels Models")
+    parser.add_argument("-v", "--verbose", action="store_const",
+                        default=1, const=2, help="Use verbose output")
+    parser.add_argument("-e", "--engine", default="all",
+                        help="Engines on which to run the test.  "
+                        "Valid values are opencl, cuda, dll, and all. "
+                        "Defaults to all if no value is given")
+    parser.add_argument("models", nargs="*",
+                        help="The names of the models to be tested.  "
+                        "If the first model is 'all', then all but the listed "
+                        "models will be tested.  See core.list_models() for "
+                        "names of other groups, such as 'py' or 'single'.")
+    opts = parser.parse_args()
+
+    if opts.engine == "opencl":
+        if not use_opencl():
+            print("opencl is not available")
+            return 1
+        loaders = ['opencl']
+    elif opts.engine == "dll":
+        loaders = ["dll"]
+    elif opts.engine == "cuda":
+        if not use_cuda():
+            print("cuda is not available")
+            return 1
+        loaders = ['cuda']
+    elif opts.engine == "all":
+        loaders = ['dll']
+        if use_opencl():
+            loaders.append('opencl')
+        if use_cuda():
+            loaders.append('cuda')
+    else:
+        print("unknown engine " + opts.engine)
+        return 1
+
+    runner = TestRunner(verbosity=opts.verbose, **test_args)
+    result = runner.run(make_suite(loaders, opts.models))
+    return 1 if result.failures or result.errors else 0
 
 
 if __name__ == "__main__":
-    sys.exit(main(*sys.argv[1:]))
+    sys.exit(main())
