@@ -32,13 +32,17 @@ Validation
 Validation of our code was done by comparing the output of the 1D model
 to the output of the software provided by the NIST (Kline, 2006).
 
-
 References
 ----------
 
 A Guinier and G. Fournet, *Small-Angle Scattering of X-Rays*,
 John Wiley and Sons, New York, (1955)
 
+Authorship and Verification
+----------------------------
+
+* **Author: P Kienzle**
+* **Last Modified by:**
 * **Last Reviewed by:** S King and P Parker **Date:** 2013/09/09 and 2014/01/06
 """
 
@@ -68,9 +72,15 @@ parameters = [["sld", "1e-6/Ang^2", 1, [-inf, inf], "",
 
 
 def form_volume(radius):
+    """Calculate volume for sphere"""
     return 1.333333333333333 * pi * radius ** 3
 
+def radius_effective(mode, radius):
+    """Calculate R_eff for sphere"""
+    return radius if mode else 0.
+
 def Iq(q, sld, sld_solvent, radius):
+    """Calculate I(q) for sphere"""
     #print "q",q
     #print "sld,r",sld,sld_solvent,radius
     qr = q * radius
@@ -104,11 +114,6 @@ def sesans(z, sld, sld_solvent, radius):
     return g
 sesans.vectorized = True  # sesans accepts an array of z values
 
-def ER(radius):
-    return radius
-
-# VR defaults to 1.0
-
 demo = dict(scale=1, background=0,
             sld=6, sld_solvent=1,
             radius=120,
