@@ -14,7 +14,7 @@ _cap_kernel(double qab, double qc, double h, double radius_cap,
 {
     // translate a point in [-1,1] to a point in [lower,upper]
     const double upper = 1.0;
-    const double lower = h/radius_cap; // integral lower bound
+    const double lower = -h/radius_cap; // integral lower bound
     const double zm = 0.5*(upper-lower);
     const double zb = 0.5*(upper+lower);
 
@@ -26,7 +26,7 @@ _cap_kernel(double qab, double qc, double h, double radius_cap,
     //    m = q R cos(theta)
     //    b = q(L/2-h) cos(theta)
     const double m = radius_cap*qc; // cos argument slope
-    const double b = (half_length-h)*qc; // cos argument intercept
+    const double b = (half_length+h)*qc; // cos argument intercept
     const double qab_r = radius_cap*qab; // Q*R*sin(theta)
     double total = 0.0;
     for (int i=0; i<GAUSS_N; i++) {
@@ -57,38 +57,23 @@ static double
 form_volume(double radius, double radius_cap, double length)
 {
     // cap radius should never be less than radius when this is called
-
-    // Note: volume V = 2*V_cap + V_cyl
-    //
-    // V_cyl = pi r_cyl^2 L
-    // V_cap = 1/6 pi h_c (3 r_cyl^2 + h_c^2) = 1/3 pi h_c^2 (3 r_cap - h_c)
-    //
-    // The docs for capped cylinder give the volume as:
-    //    V = pi r^2 L + 2/3 pi (R-h)^2 (2R + h)
-    // where r_cap=R and h = R - h_c.
-    //
-    // The first part is clearly V_cyl.  The second part requires some work:
-    //    (R-h)^2 => h_c^2
-    //    (2R+h) => 2R+ h_c-h_c + h => 2R + (R-h)-h_c + h => 3R-h_c
-    // And so:
-    //    2/3 pi (R-h)^2 (2R + h) => 2/3 pi h_c^2 (3 r_cap - h_c)
-    // which is 2 V_cap, using the second form above.
-    //
-    // In this function we are going to use the first form of V_cap
-    //
-    //      V = V_cyl + 2 V_cap
-    //        = pi r^2 L + pi hc (r^2 + hc^2/3)
-    //        = pi (r^2 (L+hc) + hc^3/3)
-    const double hc = radius_cap - sqrt(radius_cap*radius_cap - radius*radius);
-    return M_PI*(radius*radius*(length+hc) + hc*hc*hc/3.0);
+    const double h = -sqrt(square(radius_cap) - square(radius));
+    const double slice = M_PI*(square(radius_cap)*h - cube(h)/3.0);
+    const double hemisphere = 2.0*M_PI/3.0*cube(radius_cap);
+    const double rod = M_PI*square(radius)*length;
+    // h < 0 so slice is subtracted from hemisphere
+    return rod + 2.0*(hemisphere + slice);
 }
 
 static double
 radius_from_excluded_volume(double radius, double radius_cap, double length)
 {
-    const double hc = radius_cap - sqrt(radius_cap*radius_cap - radius*radius);
-    const double length_tot = length + 2.0*hc;
-    return 0.5*cbrt(0.75*radius*(2.0*radius*length_tot + (radius + length_tot)*(M_PI*radius + length_tot)));
+    const double h = -sqrt(square(radius_cap) - square(radius));
+    const double length_tot = length + 2.0*(radius_cap + h);
+    // Use cylinder excluded volume with length' = length + caps and
+    // radius' = cylinder radius since the lens is smaller than the cylinder.
+    return 0.5*cbrt(0.75*radius*(2.0*radius*length_tot
+           + (radius + length_tot)*(M_PI*radius + length_tot)));
 }
 
 static double
@@ -101,8 +86,9 @@ radius_from_volume(double radius, double radius_cap, double length)
 static double
 radius_from_totallength(double radius, double radius_cap, double length)
 {
-    const double hc = radius_cap - sqrt(radius_cap*radius_cap - radius*radius);
-    return 0.5*length + hc;
+    const double h = -sqrt(square(radius_cap) - square(radius));
+    const double half_length = 0.5*length;
+    return half_length + radius_cap - h;
 }
 
 static double
@@ -127,7 +113,7 @@ static void
 Fq(double q,double *F1, double *F2, double sld, double solvent_sld,
     double radius, double radius_cap, double length)
 {
-    const double h = sqrt(radius_cap*radius_cap - radius*radius);
+    const double h = -sqrt(square(radius_cap) - square(radius));
     const double half_length = 0.5*length;
 
     // translate a point in [-1,1] to a point in [0, pi/2]
@@ -162,7 +148,7 @@ Iqac(double qab, double qc,
     double sld, double solvent_sld, double radius,
     double radius_cap, double length)
 {
-    const double h = sqrt(radius_cap*radius_cap - radius*radius);
+    const double h = -sqrt(square(radius_cap) - square(radius));
     const double Aq = _fq(qab, qc, h, radius_cap, radius, 0.5*length);
 
     // Multiply by contrast^2 and convert to cm-1
