@@ -59,6 +59,7 @@ References
 
 .. [#] L A Feigin and D I Svergun, *Structure Analysis by Small-Angle X-Ray and
    Neutron Scattering*, Plenum Press, New York, (1987)
+.. [#] L. Onsager, *Ann. New York Acad. Sci.*, 51 (1949) 627-659
 
 Authorship and Verification
 ----------------------------
@@ -68,6 +69,7 @@ Authorship and Verification
    (corrected VR calculation)
 * **Last Reviewed by:** Paul Butler **Date:** September 06, 2018
 """
+from __future__ import division
 
 import numpy as np
 from numpy import pi, inf, sin, cos
@@ -98,40 +100,16 @@ parameters = [
 # pylint: enable=bad-whitespace, line-too-long
 
 source = ["lib/polevl.c", "lib/sas_J1.c", "lib/gauss76.c", "hollow_cylinder.c"]
-
-# pylint: disable=W0613
-def ER(radius, thickness, length):
-    """
-    :param radius:      Cylinder core radius
-    :param thickness:   Cylinder wall thickness
-    :param length:      Cylinder length
-    :return:            Effective radius
-    """
-    router = radius + thickness
-    if router == 0 or length == 0:
-        return 0.0
-    len1 = router
-    len2 = length/2.0
-    term1 = len1*len1*2.0*len2/2.0
-    term2 = 1.0 + (len2/len1)*(1.0 + 1/len2/2.0)*(1.0 + pi*len1/len2/2.0)
-    ddd = 3.0*term1*term2
-    diam = pow(ddd, (1.0/3.0))
-    return diam
-
-def VR(radius, thickness, length):
-    """
-    :param radius:      Cylinder radius
-    :param thickness:   Cylinder wall thickness
-    :param length:      Cylinder length
-    :return:            Volf ratio for P(q)*S(q)
-    """
-    router = radius + thickness
-    vol_core = pi*radius*radius*length
-    vol_total = pi*router*router*length
-    vol_shell = vol_total - vol_core
-    return vol_total, vol_shell
+have_Fq = True
+radius_effective_modes = [
+    "excluded volume", "equivalent outer volume sphere",
+    "outer radius", "half length",
+    "half outer min dimension", "half outer max dimension",
+    "half outer diagonal",
+    ]
 
 def random():
+    """Return a random parameter set for the model."""
     length = 10**np.random.uniform(1, 4.7)
     outer_radius = 10**np.random.uniform(1, 4.7)
     # Use a distribution with a preference for thin shell or thin core
@@ -153,15 +131,38 @@ demo = dict(scale=1.0, background=0.0, length=400.0, radius=20.0,
             radius_pd=.2, radius_pd_n=9,
             theta_pd=10, theta_pd_n=5,
            )
+
+def r_eff(radius, thickness, length):
+    """R_eff from excluded volume"""
+    radius += thickness
+    return (0.5*(0.75*radius*(2.0*radius*length
+                              + (radius + length)*(pi*radius + length))
+                )**(1./3.))
+
+def shell_volume(radius, thickness, length):
+    """shell volume for parameter set"""
+    return pi*((radius+thickness)**2-radius**2)*length
+
+def form_shell_ratio(radius, thickness, length):
+    """form:shell ratio"""
+    return (radius+thickness)**2/((radius+thickness)**2 - radius**2)
+
 q = 0.1
 # april 6 2017, rkh added a 2d unit test, assume correct!
 qx = q*cos(pi/6.0)
 qy = q*sin(pi/6.0)
+test_pars = [
+    parameters[0][2], # radius
+    parameters[1][2], # thickness
+    parameters[2][2], # length
+]
 # Parameters for unit tests
 tests = [
     [{}, 0.00005, 1764.926],
-    [{}, 'VR', 0.55555556],
+    [{}, 0.1, None, None,
+     r_eff(*test_pars), shell_volume(*test_pars), form_shell_ratio(*test_pars),
+    ],
     [{}, 0.001, 1756.76],
     [{}, (qx, qy), 2.36885476192],
 ]
-del qx, qy  # not necessary to delete, but cleaner
+del qx, qy, test_pars  # not necessary to delete, but cleaner
