@@ -97,7 +97,7 @@ References
 ----------
 
 .. [#Nayuk2012] R Nayuk and K Huber, *Z. Phys. Chem.*, 226 (2012) 837-854
-
+.. [#] L. Onsager, *Ann. New York Acad. Sci.*, 51 (1949) 627-659
 
 Authorship and Verification
 ----------------------------
@@ -108,7 +108,7 @@ Authorship and Verification
 """
 
 import numpy as np
-from numpy import pi, inf, sqrt
+from numpy import inf
 
 name = "hollow_rectangular_prism"
 title = "Hollow rectangular parallelepiped with uniform scattering length density."
@@ -131,7 +131,7 @@ parameters = [["sld", "1e-6/Ang^2", 6.3, [-inf, inf], "sld",
               ["sld_solvent", "1e-6/Ang^2", 1, [-inf, inf], "sld",
                "Solvent scattering length density"],
               ["length_a", "Ang", 35, [0, inf], "volume",
-               "Shorter side of the parallelepiped"],
+               "Shortest, external, size of the parallelepiped"],
               ["b2a_ratio", "Ang", 1, [0, inf], "volume",
                "Ratio sides b/a"],
               ["c2a_ratio", "Ang", 1, [0, inf], "volume",
@@ -147,37 +147,16 @@ parameters = [["sld", "1e-6/Ang^2", 6.3, [-inf, inf], "sld",
              ]
 
 source = ["lib/gauss76.c", "hollow_rectangular_prism.c"]
-
-def ER(length_a, b2a_ratio, c2a_ratio, thickness):
-    """
-    Return equivalent radius (ER)
-    thickness parameter not used
-    """
-    b_side = length_a * b2a_ratio
-    c_side = length_a * c2a_ratio
-
-    # surface average radius (rough approximation)
-    surf_rad = sqrt(length_a * b_side / pi)
-
-    ddd = 0.75 * surf_rad * (2 * surf_rad * c_side + (c_side + surf_rad) * (c_side + pi * surf_rad))
-    return 0.5 * (ddd) ** (1. / 3.)
-
-def VR(length_a, b2a_ratio, c2a_ratio, thickness):
-    """
-    Return shell volume and total volume
-    """
-    b_side = length_a * b2a_ratio
-    c_side = length_a * c2a_ratio
-    a_core = length_a - 2.0*thickness
-    b_core = b_side - 2.0*thickness
-    c_core = c_side - 2.0*thickness
-    vol_core = a_core * b_core * c_core
-    vol_total = length_a * b_side * c_side
-    vol_shell = vol_total - vol_core
-    return vol_total, vol_shell
-
+have_Fq = True
+radius_effective_modes = [
+    "equivalent cylinder excluded volume", "equivalent outer volume sphere",
+    "half length_a", "half length_b", "half length_c",
+    "equivalent outer circular cross-section",
+    "half ab diagonal", "half diagonal",
+    ]
 
 def random():
+    """Return a random parameter set for the model."""
     a, b, c = 10**np.random.uniform(1, 4.7, size=3)
     # Thickness is limited to 1/2 the smallest dimension
     # Use a distribution with a preference for thin shell or thin core
