@@ -13,21 +13,25 @@ composed of a number of uniform shells with interfacial shells between them.
 
     Example SLD profile
 
-Unlike the <onion> model (using an analytical integration), the interfacial
+Unlike the :ref:`onion` model (using an analytical integration), the interfacial
 shells here are sub-divided and numerically integrated assuming each
 sub-shell is described by a line function, with *n_steps* sub-shells per
 interface. The form factor is normalized by the total volume of the sphere.
 
+.. note::
+
+   *n_shells* must be an integer. *n_steps* must be an ODD integer.
+
 Interface shapes are as follows:
 
     0: erf($\nu z$)
-    
+
     1: Rpow($z^\nu$)
-    
+
     2: Lpow($z^\nu$)
-    
+
     3: Rexp($-\nu z$)
-    
+
     4: Lexp($-\nu z$)
 
 The form factor $P(q)$ in 1D is calculated by:
@@ -73,7 +77,6 @@ so that individual terms can be calculated as follows:
     3 \rho_\text{solvent} V(r_N)
     \Big[ \frac{\sin(qr_N) - qr_N \cos(qr_N)} {qr_N^3} \Big]
 
-
 Here we assumed that the SLDs of the core and solvent are constant in $r$.
 The SLD at the interface between shells, $\rho_{\text {inter}_i}$
 is calculated with a function chosen by an user, where the functions are
@@ -89,7 +92,7 @@ Exp:
     {\Delta t_{ \text{inter}_i }} \Big) +C  & \mbox{for } A = 0 \\
     \end{cases}
 
-Power-Law
+Power-Law:
 
 .. math::
 
@@ -102,6 +105,7 @@ Power-Law
 Erf:
 
 .. math::
+
     \rho_{{inter}_i} (r) = \begin{cases}
     B \text{erf} \Big( \frac { A(r - r_{\text{flat}_i})}
     {\sqrt{2} \Delta t_{ \text{inter}_i }} \Big) +C  & \mbox{for } A \neq 0 \\
@@ -154,7 +158,6 @@ where
     \beta_\text{in} &= qr_j \text{, } & \beta_\text{out} &= qr_{j+1}
     \end{align*}
 
-
 We assume $\rho_{\text{inter}_j} (r)$ is approximately linear
 within the sub-shell $j$.
 
@@ -177,20 +180,18 @@ where the $q$ vector is defined as
     The outer most radius is used as the effective radius for $S(Q)$
     when $P(Q) * S(Q)$ is applied.
 
-
 References
 ----------
 
 .. [#] L A Feigin and D I Svergun, Structure Analysis by Small-Angle X-Ray
    and Neutron Scattering, Plenum Press, New York, (1987)
 
-
 Authorship and Verification
-----------------------------
+---------------------------
 
 * **Author:** Jae-Hie Cho **Date:** Nov 1, 2010
 * **Last Modified by:** Paul Kienzle **Date:** Dec 20, 2016
-* **Last Reviewed by:** Paul Butler **Date:** September 8, 2018
+* **Last Reviewed by:** Steve King **Date:** March 29, 2019
 """
 
 import numpy as np
@@ -198,7 +199,7 @@ from numpy import inf, expm1, sqrt
 from scipy.special import erf
 
 name = "spherical_sld"
-title = "Sperical SLD intensity calculation"
+title = "Spherical SLD intensity calculation"
 description = """
             I(q) =
                background = Incoherent background [1/cm]
@@ -210,18 +211,20 @@ SHAPES = ["erf(|nu|*z)", "Rpow(z^|nu|)", "Lpow(z^|nu|)",
 
 # pylint: disable=bad-whitespace, line-too-long
 #            ["name", "units", default, [lower, upper], "type", "description"],
-parameters = [["n_shells",             "",           1,      [1, 10],        "volume", "number of shells"],
+parameters = [["n_shells",             "",           1,      [1, 10],        "volume", "number of shells (must be integer)"],
               ["sld_solvent",          "1e-6/Ang^2", 1.0,    [-inf, inf],    "sld", "solvent sld"],
               ["sld[n_shells]",        "1e-6/Ang^2", 4.06,   [-inf, inf],    "sld", "sld of the shell"],
               ["thickness[n_shells]",  "Ang",        100.0,  [0, inf],       "volume", "thickness shell"],
               ["interface[n_shells]",  "Ang",        50.0,   [0, inf],       "volume", "thickness of the interface"],
               ["shape[n_shells]",      "",           0,      [SHAPES],       "", "interface shape"],
-              ["nu[n_shells]",         "",           2.5,    [0, inf],       "", "interface shape exponent"],
+              ["nu[n_shells]",         "",           2.5,    [1, inf],       "", "interface shape exponent"],
               ["n_steps",              "",           35,     [0, inf],       "", "number of steps in each interface (must be an odd integer)"],
              ]
 # pylint: enable=bad-whitespace, line-too-long
 source = ["lib/polevl.c", "lib/sas_erf.c", "lib/sas_3j1x_x.c", "spherical_sld.c"]
 single = False  # TODO: fix low q behaviour
+have_Fq = True
+radius_effective_modes = ["outer radius"]
 
 profile_axes = ['Radius (A)', 'SLD (1e-6/A^2)']
 
@@ -266,14 +269,7 @@ def profile(n_shells, sld_solvent, sld, thickness,
     # return sld profile (r, beta)
     return np.asarray(z), np.asarray(rho)
 
-
-def ER(n_shells, thickness, interface):
-    """Effective radius"""
-    n_shells = int(n_shells + 0.5)
-    total = (np.sum(thickness[:n_shells], axis=1)
-             + np.sum(interface[:n_shells], axis=1))
-    return total
-
+# TODO: no random parameter generator for spherical SLD.
 
 demo = {
     "n_shells": 5,
