@@ -59,7 +59,6 @@ References
 .. [#Guinier1955] A Guinier and G. Fournet, *Small-Angle Scattering of X-Rays*, John Wiley and
    Sons, New York, (1955)
 
-
 Authorship and Verification
 ----------------------------
 
@@ -69,7 +68,7 @@ Authorship and Verification
 """
 
 import numpy as np
-from numpy import pi, inf
+from numpy import inf
 
 name = "vesicle"
 title = "Vesicle model representing a hollow sphere"
@@ -98,32 +97,11 @@ parameters = [["sld", "1e-6/Ang^2", 0.5, [-inf, inf], "sld",
              ]
 
 source = ["lib/sas_3j1x_x.c", "vesicle.c"]
-
-def ER(radius, thickness):
-    '''
-    returns the effective radius used in the S*P calculation
-
-    :param radius: core radius
-    :param thickness: shell thickness
-    '''
-    return radius + thickness
-
-def VR(radius, thickness):
-    '''
-    returns the volumes of the shell and of the whole sphere including the
-    core plus shell - is used to normalize when including polydispersity.
-
-    :param radius: core radius
-    :param thickness: shell thickness
-    :return whole: volume of core and shell
-    :return whole-core: volume of the shell
-    '''
-
-    whole = 4./3. * pi * (radius + thickness)**3
-    core = 4./3. * pi * radius**3
-    return whole, whole - core
+have_Fq = True
+radius_effective_modes = ["outer radius"]
 
 def random():
+    """Return a random parameter set for the model."""
     total_radius = 10**np.random.uniform(1.3, 5)
     radius = total_radius * np.random.uniform(0, 1)
     thickness = total_radius - radius
@@ -149,6 +127,5 @@ demo = dict(sld=0.5, sld_solvent=6.36,
 tests = [[{}, 0.0005, 859.916526646],
          [{}, 0.100600200401, 1.77063682331],
          [{}, 0.5, 0.00355351388906],
-         [{}, 'ER', 130.],
-         [{}, 'VR', 0.54483386436],
+         [{}, 0.1, None, None, 130., None, 1./0.54483386436],  # R_eff, form:shell
         ]
