@@ -310,7 +310,8 @@ def sas_2J1x_x(x):
 
 
 # Gaussians
-class Gauss:
+class Gauss(object):
+    """Gauss-Legendre integration weights"""
     def __init__(self, w, z):
         self.n = len(w)
         self.w = w
